@@ -1,6 +1,7 @@
 # Imports
 from datetime import datetime
 from pprint import pprint
+import io
 import configparser
 import boto3
 import csv
@@ -32,13 +33,17 @@ class s3_connect(object):
 
         '''
         creds = configparser.ConfigParser()
-        creds.read(config_file)
-        self.DEFAULT_BUCKET=creds.get(section, 'default_bucket')
-
+        self.config_file = config_file
+        self.section = section
+        creds.read(self.config_file)
+        self.DEFAULT_BUCKET=creds.get(self.section, 'default_bucket')
+        self.access_key = creds.get(self.section, 'aws_access_key_id')
+        self.secret_key = creds.get(self.section, 'aws_secret_access_key')
+        
         self.conn = boto3.resource(
             's3',
-            aws_access_key_id=creds.get(section, 'aws_access_key_id'),
-            aws_secret_access_key=creds.get(section, 'aws_secret_access_key'),
+            aws_access_key_id=self.access_key,
+            aws_secret_access_key=self.secret_key,
         )
 
 
@@ -77,17 +82,8 @@ class s3_connect(object):
     def upload_model(self, model, file_location, model_type, bucket=None, pkl_model_type='', append_ts=False, suffix='csv'):
         '''
         bucket = optional, defaults to 'default_bucket' section in config file
-        model = ('complaints_first_send',
-                'complaints_next_send',
-                'offer_recommender_v2_1',
-                'offer_recommender_v2_2',
-                'offer_recommender_v2_3',
-                'complaint_model_v2_0',
-                'complaint_model_v2_1',
-                'reactivation_model_v2_0',
-                'reactivation_model_v2_1',
-                'reactivation_model_v2_2',
-                'conversion_model_v1_0')
+        
+        model = comma delimited list of valid model names in .databases.conf file
 
         file_location = path to file
 
@@ -101,19 +97,10 @@ class s3_connect(object):
         returns dict {'bucket': <bucket>, 'key': <key>}
 
         '''
-        valid_models = (
-            'complaints_first_send',
-            'complaints_next_send',
-            'offer_recommender_v2_1',
-            'offer_recommender_v2_2',
-            'offer_recommender_v2_3',
-            'complaint_model_v2_0',
-            'complaint_model_v2_1',
-            'reactivation_model_v2_0',
-            'reactivation_model_v2_1',
-            'reactivation_model_v2_2',
-            'conversion_model_v1_0'
-            )
+        creds = configparser.ConfigParser()
+        self.config_file = config_file
+        creds.read(self.config_file)
+        valid_models = tuple(creds.get(self.section, 'model'))
 
         types = ('input', 'output', 'pickled_model')
 
@@ -183,21 +170,55 @@ class s3_connect(object):
         print(self.conn.meta.client.copy(copy_source, bucket, dest_key))
         return 's3://{0}/{1}'.format(bucket, dest_key)
 
+
     def download_file(self, key, dest_file=None, bucket=None):
         '''
         key = s3 key
         dest_file = optional path to desired file output
         bucket = optional, defaults to 'default_bucket' section in config file
         '''
+
+        filename = key.split('/')[-1]
+
         if bucket == None:
             bucket = self.DEFAULT_BUCKET
+
         if dest_file == None:
-            dest_file = key.split('/')[-1]
+            path = key.split('/')[-1]
+        else:
+            path = os.path.join(dest_file, filename)
+
 
         bucket = self.conn.Bucket(bucket)
-        bucket.download_file(key, dest_file)
+        bucket.download_file(key, path)
 
-        return dest_file
+        return path
+
+
+    def get_contents(self, key, bucket=None, stringIO=False):
+        '''
+        key <string> - s3 key
+
+        bucket <string> - optional, defaults to 'default_bucket' section in config file
+
+        stringIO <boolean> - optional, will return stringIO stream
+
+        returns BytesIO stream
+        '''
+
+
+        io_bytes = io.BytesIO()
+        if bucket == None:
+            bucket = self.DEFAULT_BUCKET
+
+        bucket = self.conn.Bucket(bucket)
+        bucket.download_fileobj(key, io_bytes)
+
+        if stringIO == True:
+            io_string = io.StringIO(io_bytes.getvalue().decode('utf-8'))
+            return io_string
+
+        return io_bytes
 
 
     def del_key(self, key, bucket=None):
